package main

import (
  "fmt"
  "io/ioutil"
  "os"
  "strconv"
  "strings"
  "sort"
)

type Carrello struct {
  pos int
  carico int
}

func (c Carrello) String() string {
  return fmt.Sprintf("posizione %d, carico %d", c.pos, c.carico)
}

func aggiornaStato(c *Carrello, posizione, carico int) bool {
  if posizione < 0 || carico < 0 {
    return false
  }
  c.pos = posizione
  c.carico = carico
  return true
}

/* Dato il contenuto del file che contiene il  percorso, lo trasforma in una mappa in cui le chiavi
sono le posizioni (0,1,2,3,...) dove c'è qualcosa e i valori sono i contenuti. Il secondo valore restituito
è il numero di celle del percorso */
func filecontent2map(s string) (map[int]int,int) {
  // NB: il primo (|) e gli ultimi due (|\n) caratteri della stringa vanno eliminati perché non rappresentano separatori di celle
  sls := strings.Split(s[1:len(s)-2], "|") // Divide il percorso in una slice di stringhe che saranno o spazi o numeri
  m := make(map[int]int)
  for i, s := range sls {
    if s != " " {
      n, _ := strconv.Atoi(s)
      m[i] = n
    }
  }
  return  m, len(sls)
}

/* Stampa la situazione del percorso, dove m descrive i contenuti delle varie celle e nCelle è il numero di celle. */
func stampaPercorso(m map[int]int, nCelle int) {
  fmt.Printf("|")
  for i := 0 ; i < nCelle; i++ {
    if m[i] == 0 {
      fmt.Printf(" |")
    } else {
      fmt.Printf("%d|", m[i])
    }
  }
  fmt.Printf("\n")
}

func main() {
  // Leggo il file
  b,err := ioutil.ReadFile(os.Args[1])
  if err !=  nil {
    fmt.Fprintf(os.Stderr, "%v", err)
    os.Exit(1)
  }
  // Creo un percorso (rappresentato da una mappa e da un numero di celle)
  s := string(b)
  m, nCelle := filecontent2map(s)
  stampaPercorso(m, nCelle)  // Stampo il  percorso all'inizio

  var c Carrello    // Il carrello
  var peso2numero map[int]int   //Per ogni peso trovato, quanti oggetti di quel peso sono stati trovati
  var pesi []int    // Pesi trovati (tutti distinti)

  peso2numero = make(map[int]int)
  nViaggi := 0    // Numero di viaggi effettuati dal carrello
  maxPeso := 0    // Massimo peso trovato fin qui
  for {
    if c.pos == nCelle { // Lavoro finito!
      fmt.Printf("carrello: %v\n", c)   // Stampo il carrello
      nViaggi++
      m[0] += c.carico
      aggiornaStato(&c, 0, 0)   // Torno a casa e svuoto il carrello
      stampaPercorso(m, nCelle) // Stampe finali
      fmt.Printf("carrello: %v\n", c)
      break  // Fine del ciclo di lavoro
    }
    if m[c.pos] > 0 {   // Ho trovato un oggetto
        if m[c.pos] > maxPeso {  // E' il più pesante trovato fino ad ora?
          maxPeso = m[c.pos]
        }
        if m[c.pos] + c.carico <= 15 {  // Se posso caricarlo
          if peso2numero[m[c.pos]] == 0 {  // E' un peso mai trovato prima?
            pesi = append(pesi, m[c.pos])
          }
          peso2numero[m[c.pos]]++  // Incremento il numero di oggetti di quel peso trovati
          delete(m,c.pos)
          aggiornaStato(&c, c.pos + 1, c.carico + m[c.pos])  // Carico l'oggetto e lo tolgo dal percorso
        } else {
          fmt.Printf("carrello: %v\n", c)  // Se non posso caricarlo
          nViaggi++
          m[0] += c.carico
          aggiornaStato(&c, 1, 0)  // Torno a casa e svuoto il carrello
          stampaPercorso(m, nCelle)
        }
    } else {  // Non c'è niente nella cella: mi sposto semplicemente a destra
      c.pos++
    }
  }

  // Stampe finali
  fmt.Printf("n viaggi: %d\n", nViaggi)
  fmt.Printf("peso max: %d\n", maxPeso)
  sort.Ints(pesi)   // Ordino i pesi
  for _, peso := range pesi {
    fmt.Printf("%d ogg. di peso %d\n", peso2numero[peso], peso)
  }
}
